@@ -1,6 +1,7 @@
 import databases
 import os
 
+from fastapi import Depends, Response
 from fastapi_users import FastAPIUsers
 from fastapi_users.authentication import CookieAuthentication, JWTAuthentication
 from msdss_base_api import API
@@ -56,6 +57,8 @@ class UsersAPI(API):
         Whether to setup a startup event to connect databases.
     setup_shutdown : bool
         Whether to setup a shutdown event to close databases.
+    setup_jwt_refresh : bool
+        Whether to setup a token refresh route at ``jwt_refresh_path``. This is setup according to the FastAPI Users `JWT authentication guide <https://fastapi-users.github.io/fastapi-users/configuration/authentication/jwt/>`_.
     auth_router_auth: 'jwt' or 'cookie'
         FastAPI Users auth method for the auth router. See `Auth router <https://fastapi-users.github.io/fastapi-users/configuration/routers/auth/>`_.
     auth_router_kwargs : dict
@@ -78,7 +81,7 @@ class UsersAPI(API):
         Keyword arguments passed to :meth:`fastapi_users:fastapi_users.FastAPIUsers.get_users_router`. See `Users router <https://fastapi-users.github.io/fastapi-users/configuration/routers/users/>`_.
     users_router_include_kwargs : dict
         Keyword arguments passed to :meth:`fastapi:fastapi.FastAPI.include_router` for the FastAPI Users users router.
-    reset_password_token_secret : str,
+    reset_password_token_secret : str
         Secret used to secure password reset tokens. If ``None``, it will default to param ``secret``.
     verification_token_secret : str or None
         Secret used to secure verification tokens. If ``None``, it will default to param ``secret``.
@@ -90,6 +93,8 @@ class UsersAPI(API):
         Secret used to secure JWT tokens. If ``None``, it will default to param ``secret``.
     jwt_kwargs : dict
         Dictionary of keyword arguments passed to :class:`fastapi_users:fastapi_users.authentication.JWTAuthentication`.  See `JWTAuthentication <https://fastapi-users.github.io/fastapi-users/configuration/authentication/jwt/>`_.
+    jwt_refresh_path : str
+        Path to setup refresh token if ``setup_jwt_refresh`` is ``True``.
     Base : class
         Class returned from :func:`sqlalchemy:sqlalchemy.orm.declarative_base`.
     User : :class:`msdss_users_api.models.User`
@@ -102,14 +107,16 @@ class UsersAPI(API):
         UserDB model for FastAPI Users. See :class:`msdss_users_api.models.UserDB`.
     UserTable : :class:`msdss_users_api.models.UserTable`
         UserTable model for FastAPI Users. See :class:`msdss_users_api.models.UserTable`.
+    UserManager : :class:`msdss_users_api.models.UserManager`
+        UserManager model for FastAPI Users. See :class:`msdss_users_api.models.UserManager`.
     database_engine : :func:`sqlalchemy:sqlalchemy.create_engine` or  None
         SQLAlchemy engine object. If ``None``, one will be created from the database connection params.
     async_database : :class:`databases:databases.Database` or None
         Async database object from ``databases``. If ``None``, one will be created from database connection params.
     get_user_db : func or None
-        Function for getting the FastAPI Users DB object. If ``None``, one will be setup from params. See ``FastAPI Users database config <https://fastapi-users.github.io/fastapi-users/configuration/databases/sqlalchemy/>``_.
+        Function for getting the FastAPI Users DB object. If ``None``, one will be setup from params. See ``FastAPI Users database config <https://fastapi-users.github.io/fastapi-users/configuration/databases/sqlalchemy/>`_.
     get_user_manager : func or None
-        Function for getting the FastAPI UserManager object. If ``None``, one will be setup from params. See ``FastAPI Users UserManager <https://fastapi-users.github.io/fastapi-users/configuration/user-manager/>``_.
+        Function for getting the FastAPI UserManager object. If ``None``, one will be setup from params. See ``FastAPI Users UserManager <https://fastapi-users.github.io/fastapi-users/configuration/user-manager/>`_.
     cookie_auth : :class:`fastapi_users:fastapi_users.authentication.CookieAuthentication` or None
         A cookie authentication object from FastAPI Users. See `CookieAuthentication <https://fastapi-users.github.io/fastapi-users/configuration/authentication/cookie/>`_.
     jwt_auth : :class:`fastapi_users:fastapi_users.authentication.JWTAuthentication` or None
@@ -178,7 +185,7 @@ class UsersAPI(API):
         )
 
         # Run the app with app.start()
-        # API is hosted at http://localhost:8000
+        # Try API at http://localhost:8000/docs
         # app.start()
     """
     def __init__(
@@ -200,6 +207,7 @@ class UsersAPI(API):
         use_cookie_auth=False,
         setup_startup=True,
         setup_shutdown=True,
+        setup_jwt_refresh=True,
         auth_router_auth='jwt',
         auth_router_kwargs={},
         auth_router_include_kwargs={
@@ -235,12 +243,14 @@ class UsersAPI(API):
             'lifetime_seconds': 3600,
             'tokenUrl': 'auth/jwt/login'
         },
+        jwt_refresh_path='/auth/jwt/refresh',
         Base=Base,
         User=User,
         UserCreate=UserCreate,
         UserUpdate=UserUpdate,
         UserDB=UserDB,
         UserTable=UserTable,
+        UserManager=None,
         database_engine=None,
         async_database=None,
         get_user_db=None,
@@ -309,7 +319,8 @@ class UsersAPI(API):
         
         # (UserAPI_func) Setup required functions
         get_user_db = create_user_db_func(database_engine, async_database, sqlalchemy_base=Base, user_table_model=UserTable, user_db_model=UserDB) if get_user_db is None else get_user_db
-        get_user_manager = create_user_manager_func(get_user_db, secret, reset_password_token_secret=reset_password_token_secret, verification_token_secret=verification_token_secret) if get_user_manager is None else get_user_manager
+        UserManager = create_user_manager_model(secret, reset_password_token_secret=reset_password_token_secret, verification_token_secret=verification_token_secret) if UserManager is None else UserManager
+        get_user_manager = create_user_manager_func(get_user_db, UserManager) if get_user_manager is None else get_user_manager
 
         # (UserAPI_api) Setup users api obj
         users_api = FastAPIUsers(
@@ -362,7 +373,66 @@ class UsersAPI(API):
             async def during_shutdown():
                 await async_database.disconnect()
 
+        # (UserAPI_route_refresh) Setup refresh token path
+        if setup_jwt_refresh:
+            @self.add('POST', jwt_refresh_path)
+            async def refresh_jwt(response: Response, user=Depends(users_api.current_user(active=True))):
+                return await auth_router_auth.get_login_response(user, response, UserManager)
+
         # (UserAPI_attr) Add attributes
         self.users_api = users_api
         self._database_engine = database_engine
-        self._async_database = async_database
\ No newline at end of file
+        self._async_database = async_database
+
+    def get_current_user(self, *args, **kwargs):
+        """
+        Get a dependency function to retrieve the current authenticated user.
+
+        Parameters
+        ----------
+        *args, **kwargs
+            Additional arguments passed to :meth:`fastapi_users:fastapi_users.FastAPIUsers.current_user`. See `current_user <https://fastapi-users.github.io/fastapi-users/usage/current-user/>`_.
+
+        Return
+        ------
+        func
+            A function to retrieve the current authenticated user. Useful for adding protected routes accessible only by authenticated users.
+
+        Author
+        ------
+        Richard Wen <rrwen.dev@gmail.com>
+
+        Example
+        -------
+        .. jupyter-execute::
+            :hide-output:
+
+            from fastapi import Depends
+            from msdss_users_api import UsersAPI
+            from msdss_users_api.models import User
+
+            app = UsersAPI(
+                secret='secret-phrase',
+                jwt_secret='secret-phrase-02',
+                driver='postgresql',
+                user='msdss',
+                password='msdss123',
+                host='localhost',
+                port='5432',
+                database='msdss'
+            )
+
+            # Get a function dependency for the current active user
+            current_active_user = app.get_current_user(active=True)
+
+            # Add a protected route
+            @app.add('GET', '/protected-route')
+            def protected_route(user: User = Depends(current_active_user)):
+                return f'Hello, {user.email}'
+
+            # Run the app with app.start()
+            # Try API at http://localhost:8000/docs
+            # app.start()
+        """
+        out = self.users_api.current_user(*args, **kwargs)
+        return out
\ No newline at end of file
